==> labda-0.0.7-py3-none-any.whl: labda/structure/validation/subject.py <==
from enum import StrEnum

import pandera as pr


class Column(StrEnum):
    SUBJECT_ID = "subject_id"
    DATETIME = "datetime"
    WEAR = "wear"
    TIMEDELTA = "timedelta"
    # ---
    VERTICAL_COUNTS = "vertical_counts"
    HORIZONTAL_COUNTS = "horizontal_counts"
    PERPENDICULAR_COUNTS = "perpendicular_counts"
    VM_COUNTS = "vm_counts"
    POSITION = "position"
    STEPS = "steps"
    MOTION = "motion"
    # ---
    LATITUDE = "latitude"
    LONGITUDE = "longitude"
    GNSS_ACCURACY = "gnss_accuracy"
    NSAT_VIEWED = "nsat_viewed"
    NSAT_USED = "nsat_used"
    NSAT_RATIO = "nsat_ratio"
    SNR_VIEWED = "snr_viewed"
    SNR_USED = "snr_used"
    PDOP = "pdop"
    HDOP = "hdop"
    VDOP = "vdop"
    # ---
    DISTANCE = "distance"
    ELEVATION = "elevation"
    SPEED = "speed"
    ACCELERATION = "acceleration"
    DIRECTION = "direction"
    ENVIRONMENT = "environment"
    # ---
    SEGMENT_ID = "segment_id"
    TRIP_ID = "trip_id"
    TRIP_STATUS = "trip_status"
    TRIP_MODE = "trip_mode"
    STATIONARY_ID = "stationary_id"
    # ---
    HEART_RATE = "heart_rate"
    LUX = "lux"
    # ---
    ACTIVITY_INTENSITY = "activity_intensity"
    ACTIVITY_VALUE = "activity_value"
    ACTIVITY = "activity"
    # ---
    CONTEXT = "context"


SCHEMA = pr.DataFrameSchema(
    index=pr.Index(
        name=Column.DATETIME,
        dtype="datetime64[ns]",
        description="Datetime in timezone where data was collected.",
    ),
    columns={
        Column.SUBJECT_ID: pr.Column(
            dtype="string",
            description="Unique ID of the subject.",
            required=False,
            nullable=False,
        ),
        Column.WEAR: pr.Column(
            dtype="boolean",
            description="Devices worn or not.",
            required=False,
            nullable=True,
        ),
        Column.TIMEDELTA: pr.Column(
            dtype="timedelta",
            description="Time since last fix.",
            required=False,
            nullable=True,
        ),
        Column.VERTICAL_COUNTS: pr.Column(
            dtype="UInt16",
            description="Vertical counts.",
            required=False,
            nullable=True,
        ),
        Column.HORIZONTAL_COUNTS: pr.Column(
            dtype="UInt16",
            description="Horizontal counts.",
            required=False,
            nullable=True,
        ),
        Column.PERPENDICULAR_COUNTS: pr.Column(
            dtype="UInt16",
            description="Perpendicular counts.",
            required=False,
            nullable=True,
        ),
        Column.VM_COUNTS: pr.Column(
            dtype="Float32",
            description="Vector magnitude based on axes counts.",
            required=False,
            nullable=True,
        ),
        Column.POSITION: pr.Column(
            dtype="category",
            description="Position of subject.",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s[s.notna()].isin(
                        [
                            "sitting",
                            "sitting-lying",
                            "lying",
                            "standing",
                        ]
                    ),
                )
            ],
        ),
        Column.MOTION: pr.Column(
            dtype="boolean",
            description="Motion detected or not.",
            required=False,
            nullable=True,
        ),
        Column.STEPS: pr.Column(
            dtype="UInt16",
            description="Steps since last fix.",
            required=False,
            nullable=True,
        ),
        Column.LATITUDE: pr.Column(
            dtype="Float64",
            description="Latitude in appropriate coordinate system where data was collected.",
            required=False,
            nullable=True,
            # checks=[Check.ge(-90), Check.le(90)],
        ),
        Column.LONGITUDE: pr.Column(
            dtype="Float64",
            description="Longitude in appropriate coordinate system where data was collected.",
            required=False,
            nullable=True,
            # checks=[Check.ge(-180), Check.le(180)],
        ),
        Column.GNSS_ACCURACY: pr.Column(
            dtype="Float32",
            description="Accuracy of GNSS fix in metres.",
            required=False,
            nullable=True,
        ),
        Column.NSAT_VIEWED: pr.Column(
            dtype="UInt8",
            description="Number of satellites viewed by device.",
            required=False,
            nullable=True,
        ),
        Column.NSAT_USED: pr.Column(
            dtype="UInt8",
            description="Number of satellites used by device.",
            required=False,
            nullable=True,
        ),
        Column.NSAT_RATIO: pr.Column(
            dtype="Float32",
            description="Satellites ratio (used/viewed).",
            required=False,
            nullable=True,
        ),
        Column.SNR_VIEWED: pr.Column(
            dtype="UInt16",
            description="Sum of satellites SNR.",
            required=False,
            nullable=True,
        ),
        Column.SNR_USED: pr.Column(
            dtype="UInt16",
            description="Sum of used satellites SNR.",
            required=False,
            nullable=True,
        ),
        Column.PDOP: pr.Column(
            dtype="Float32",
            description="GNSS PDOP (Position of Dilution of Precision).",
            required=False,
            nullable=True,
        ),
        Column.HDOP: pr.Column(
            dtype="Float32",
            description="GNSS HDOP (Horizontal Dilution of Precision).",
            required=False,
            nullable=True,
        ),
        Column.VDOP: pr.Column(
            dtype="Float32",
            description="GNSS VDOP (Vertical Dilution of Precision).",
            required=False,
            nullable=True,
        ),
        Column.ACCELERATION: pr.Column(
            dtype="Float32",
            description="Acceleration in m/s².",
            nullable=True,
            required=False,
        ),
        Column.DIRECTION: pr.Column(
            dtype="Float32",
            description="Direction (bearing) in degrees",
            required=False,
            nullable=True,
        ),
        Column.DISTANCE: pr.Column(
            dtype="Float32",
            description="Distance taken since last fix in metres.",
            required=False,
            nullable=True,
        ),
        Column.ELEVATION: pr.Column(
            dtype="Float32",
            description="Elevation in metres",
            required=False,
            nullable=True,
        ),
        Column.ENVIRONMENT: pr.Column(
            dtype="category",
            description="Type of environment",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s[s.notna()].isin(
                        [
                            "indoor",
                            "outdoor",
                            "vehicle",
                            "unknown",
                        ]
                    ),
                )
            ],
        ),
        Column.SPEED: pr.Column(
            dtype="Float32",
            description="Speed in km/h.",
            required=False,
            nullable=True,
        ),
        Column.SEGMENT_ID: pr.Column(
            dtype="UInt16",
            description="ID of segment based on missing data or other criteria.",
            required=False,
            nullable=False,
        ),
        Column.TRIP_ID: pr.Column(
            dtype="UInt16",
            description="ID of trip.",
            required=False,
            nullable=True,
        ),
        Column.TRIP_MODE: pr.Column(
            dtype="category",
            description="Type of transportation mode.",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s[s.notna()].isin(
                        [
                            "walk",
                            "run",
                            "walk/run",
                            "bicycle",
                            "scooter",
                            "vehicle",
                            "train",
                        ]
                    ),
                )
            ],
        ),
        Column.TRIP_STATUS: pr.Column(
            dtype="category",
            description="Status of trip",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s[s.notna()].isin(
                        [
                            "start",
                            "end",
                            "start-end",
                            "transport",
                            "pause",
                            "stationary",
                        ]
                    ),
                )
            ],
        ),
        Column.STATIONARY_ID: pr.Column(
            dtype="UInt16",
            description="ID of stationary.",
            required=False,
            nullable=True,
        ),
        Column.HEART_RATE: pr.Column(
            dtype="UInt8",
            description="Heart rate in beats per minute.",
            required=False,
            nullable=True,
        ),
        Column.LUX: pr.Column(
            dtype="UInt16",
            description="Ambient light in lux.",
            required=False,
            nullable=True,
        ),
        Column.ACTIVITY_VALUE: pr.Column(
            dtype="Float32",
            description="Vendor specific activity value (metric).",
            required=False,
            nullable=True,
        ),
        Column.ACTIVITY: pr.Column(
            dtype="category",
            description="Type of activity.",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s[s.notna()].isin(
                        [
                            "resting",
                            "sporadic_walking",
                            "walking",
                            "jogging",
                            "running",
                            "bicycling",
                        ]
                    ),
                )
            ],
        ),
        Column.ACTIVITY_INTENSITY: pr.Column(
            dtype="category",
            description="Intensity of activity.",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s[s.notna()].isin(
                        [
                            "sedentary",
                            "light",
                            "moderate",
                            "moderate-vigorous",
                            "vigorous",
                            "very_vigorous",
                        ]
                    ),
                )
            ],
        ),
        Column.CONTEXT: pr.Column(
            dtype="category",
            description="Spatiotemporal context.",
            required=False,
            nullable=False,
        ),
    },
    coerce=True,
    strict=False,
)

==> labda-0.0.11-py3-none-any.whl: labda/structure/validation/subject.py <==
from enum import StrEnum

import pandera as pr


class SampleRateUnit(StrEnum):
    HERTZ = "hz"
    SECOND = "s"


class Vendor(StrEnum):
    ACTIGRAPH = "ActiGraph"
    AXIVITY = "Axivity"
    QSTARZ = "Qstarz"
    SENS = "Sens"
    TRACCAR = "Traccar"


class Placement(StrEnum):
    THIGH = "thigh"
    HIP = "hip"
    WRIST = "wrist"
    MOBILE = "mobile"


class Column(StrEnum):
    # --- Base ---
    SUBJECT_ID = "subject_id"
    DATETIME = "datetime"
    TIMEDELTA = "timedelta"
    WEAR = "wear"

    # --- Accelerometry ---
    ACC_X = "acc_x"
    ACC_Y = "acc_y"
    ACC_Z = "acc_z"
    ACC_VM = "acc_vm"

    # --- Counts ---
    COUNTS_X = "counts_x"
    COUNTS_Y = "counts_y"
    COUNTS_Z = "counts_z"
    COUNTS_VM = "counts_vm"

    # --- Taxonomy ---
    MOVEMENT = "movement"
    ACTION = "action"
    ACTIVITY = "activity"

    # --- Other ---
    ACTIVITY_INTENSITY = "activity_intensity"
    ACTIVITY_VALUE = "activity_value"
    STEPS = "steps"
    HEART_RATE = "heart_rate"
    TEMPERATURE = "temperature"
    LUX = "lux"

    # --- Spatial ---
    LATITUDE = "latitude"
    LONGITUDE = "longitude"
    GNSS_ACCURACY = "gnss_accuracy"
    NSAT_VIEWED = "nsat_viewed"
    NSAT_USED = "nsat_used"
    NSAT_RATIO = "nsat_ratio"
    SNR_VIEWED = "snr_viewed"
    SNR_USED = "snr_used"
    PDOP = "pdop"
    HDOP = "hdop"
    VDOP = "vdop"
    DISTANCE = "distance"
    ELEVATION = "elevation"
    SPEED = "speed"
    ACCELERATION = "acceleration"
    DIRECTION = "direction"
    ENVIRONMENT = "environment"

    # --- Trips and Contexts ---
    SEGMENT_ID = "segment_id"
    TRIP_ID = "trip_id"
    TRIP_STATUS = "trip_status"
    TRIP_MODE = "trip_mode"
    STATIONARY_ID = "stationary_id"
    CONTEXT = "context"


SCHEMA = pr.DataFrameSchema(
    index=pr.Index(
        name=Column.DATETIME,
        dtype="datetime64[ns]",
    ),
    columns={
        # --- Base ---
        Column.SUBJECT_ID: pr.Column(
            dtype="string",
            required=False,
            nullable=False,
        ),
        Column.TIMEDELTA: pr.Column(
            dtype="timedelta",
            required=False,
            nullable=True,
        ),
        Column.WEAR: pr.Column(
            dtype="boolean",
            required=False,
            nullable=True,
        ),
        # --- Accelerometry ---
        Column.ACC_X: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.ACC_Y: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.ACC_Z: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.ACC_VM: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        # --- Counts ---
        Column.COUNTS_X: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.COUNTS_Y: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.COUNTS_Z: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.COUNTS_VM: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        # --- Taxonomy ---
        Column.MOVEMENT: pr.Column(
            dtype="string",
            required=False,
            nullable=True,
        ),  # TODO: Add categories (Bodily movement defined by anatomy and biomechanics)
        Column.ACTION: pr.Column(
            dtype="string",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s.isin(
                        [
                            "lying",
                            "lying-sitting",  # Extra class from Sens
                            "reclining",
                            "sitting",
                            "standing",
                            "kneeling",
                            "shuffling",
                            "walking",
                            "ascending (stairs)",
                            "descending (stairs)",
                            "crawling",
                            "dancing",
                            "jumping",
                            "running",
                            "diving",
                            "cycling",
                            "swimming",
                            "climbing",
                            "gliding (skates, skis)",
                            "lying down",
                            "reclining up",
                            "reclining down",
                            "sitting up",
                            "sitting down",
                            "standing up",
                            "turning left",
                            "turning right",
                            "kneeling down",
                        ]
                    ),
                )
            ],
        ),
        Column.ACTIVITY: pr.Column(
            dtype="string",
            required=False,
            nullable=True,
        ),  # TODO: Add categories (HETUS)
        # --- Other ---
        Column.ACTIVITY_INTENSITY: pr.Column(
            dtype="category",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s.isin(
                        [
                            "sedentary",
                            "light",
                            "moderate",
                            "moderate-vigorous",
                            "vigorous",
                            "very vigorous",
                        ]
                    ),
                )
            ],
        ),
        Column.ACTIVITY_VALUE: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.STEPS: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.HEART_RATE: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.TEMPERATURE: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.LUX: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        # --- Spatial ---
        Column.LATITUDE: pr.Column(
            dtype="Float64",
            required=False,
            nullable=True,
        ),
        Column.LONGITUDE: pr.Column(
            dtype="Float64",
            required=False,
            nullable=True,
        ),
        Column.GNSS_ACCURACY: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.NSAT_VIEWED: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.NSAT_USED: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.NSAT_RATIO: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.SNR_VIEWED: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.SNR_USED: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.PDOP: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.HDOP: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.VDOP: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.DISTANCE: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.ELEVATION: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.SPEED: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.ACCELERATION: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.DIRECTION: pr.Column(
            dtype="Float32",
            required=False,
            nullable=True,
        ),
        Column.ENVIRONMENT: pr.Column(
            dtype="category",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s.isin(
                        [
                            "indoor",
                            "outdoor",
                            "mixed",
                        ]
                    ),
                )
            ],
        ),
        # --- Trips and Contexts ---
        Column.SEGMENT_ID: pr.Column(
            dtype="UInt16",
            required=False,
            nullable=True,
        ),
        Column.TRIP_STATUS: pr.Column(
            dtype="category",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s.isin(
                        [
                            "start",
                            "end",
                            "start-end",
                            "transport",
                            "pause",
                            "stationary",
                        ]
                    ),
                )
            ],
        ),
        Column.TRIP_ID: pr.Column(
            dtype="UInt16",
            required=False,
            nullable=True,
        ),
        Column.STATIONARY_ID: pr.Column(
            dtype="UInt16",
            required=False,
            nullable=True,
        ),
        Column.TRIP_MODE: pr.Column(
            dtype="category",
            required=False,
            nullable=True,
            checks=[
                pr.Check(
                    lambda s: s.isin(
                        [
                            "walk",
                            "run",
                            "walk/run",
                            "bicycle",
                            "vehicle",
                            "public transport",
                            "train",
                        ]
                    ),
                )
            ],
        ),
        Column.CONTEXT: pr.Column(
            dtype="category",
            required=False,
            nullable=True,
        ),
    },
    coerce=True,
    strict=False,
)
